@@ -22,9 +22,11 @@ setMethod("itemUnion", signature(x = "itemMatrix", y = "itemMatrix"),
   function(x, y) {
     if(length(x)!=length(y)) stop("Length mismatch between x and y!")
     
-    x@data <- .Call("R_or_ngCMatrix", x@data, y@data, 
-      PACKAGE="arules")
-    #x@data <- as(x@data+y@data, "ngCMatrix")
+    ### the C code does not deal well with a large number of dense rules.
+    #x@data <- .Call("R_or_ngCMatrix", x@data, y@data, 
+    #  PACKAGE="arules")
+    
+    x@data <- as(x@data+y@data, "ngCMatrix")
     
     x
   }
